from classes.Clause import *
from functions import subtrees, sort_clauses, contains_verb

SUBJECTS = ["nsubj", "nsubjpass", "csubj", "csubjpass", "agent", "expl"]
COMP = ["compound", "amod", "conj"]

class Sentence:
  def __init__(self, paragraphID, _id, nlp, stopwords, constraint, original): 
    self.paragraphID = paragraphID
    self._id = _id
    self.nlp = nlp
    self.stopwords = stopwords
    self.constraint = constraint
    self.original = original.as_doc()
    self.clauses = self.calculate_clauses()

    
  def __repr__(self):
    s = "ORIGINAL: " + self.original.text + "\n"
    for cl in self.clauses:
      s += repr(cl) + "\n\n"  
    return s
    
  def __str__(self):
    s = "ORIGINAL: " + self.original.text + "\n"
    for cl in self.clauses:
      s += repr(cl) + "\n\n"  
    return s
  
  def __add__(self, other):
    return str(self) + other
  
  def __radd__(self, other):
    return other + str(self)
    
        
  def calculate_clauses(self):
    for word in self.original:
      if(word.dep_ == 'ROOT'):
        root = word       
    trees = subtrees(root)   #cf. functions.py
    curr = list()
    clauses = list()
    for idx, t in enumerate(reversed(trees)):
      if(t[0].dep_ == "ROOT" or (t[0].dep_ == 'conj' and t[0].pos_ == "VERB") or (t[0].dep_ == 'prep' and contains_verb(t[1]))): 
        clauses.append(Clause(self._id + str(idx), self.nlp, self.stopwords, [i for i in t[1] if not i in curr]))
        curr.extend(t[1])
    clauses.sort(key = sort_clauses)
    #subject cannot be set within clause as it might be that we need the subject of the previous clause -> anaphora resolution
    previous_subj = None
    for clause in clauses:   
      for token in clause.clause:
        if(token.dep_ in SUBJECTS and token.head.pos_ == 'VERB' and token.pos_ != "PRON"):
          if(token.dep_ == "agent"):
            previous_subj = [t for t in token.subtree if ((t.dep_ in COMP and t in token.lefts) or t == token)]
          else:
            previous_subj = [t for t in token.subtree if ((t.dep_ in COMP and t in token.lefts) or t == token)]
      clause.set_subject(previous_subj)
      #set remaining elements -> needed for cost_score
      clause.set_remaining()
    return clauses  
 
